set linesize 32000
set pagesize 1000
set echo on
set termout on
set trimspool on

spool joinhints.log

set define off

drop table sales;
drop table products;
drop table customers;

create table sales
(
sale_date date,
product_number number,
customer_number number,
amount number
);

create table products
(
product_number number,
product_type varchar2(12),
product_name varchar2(12)
);

create table customers
(
customer_number number,
customer_state varchar2(14),
customer_name varchar2(17)
);

insert into sales values (to_date('01/02/2012','MM/DD/YYYY'),1,1,100);
insert into sales values (to_date('01/03/2012','MM/DD/YYYY'),1,2,200);
insert into sales values (to_date('01/04/2012','MM/DD/YYYY'),2,1,300);
insert into sales values (to_date('01/05/2012','MM/DD/YYYY'),2,2,400);

insert into products values (1,'Cheese','Chedder');
insert into products values (2,'Cheese','Feta');

insert into customers values (1,'FL','Sunshine State Co');
insert into customers values (2,'FL','Green Valley Inc');

set autotrace on

-- wrong way of doing join hints
-- assumes I'm listing outer and inner
-- or left and right side of join in
-- use_nl and use_hash hints

select /*+ leading(sales products) use_hash(sales products)
use_nl(products customers) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

-- still wrong.  assumes use_hash and use_nl
-- have the outer or left side of the join

select /*+ leading(sales products) use_hash(sales)
use_nl(products) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

-- correct.  sales and product use hash join with
-- product the inner or right table in the join
-- customers the inner table on a nested loops join
-- from the resulte of the sales-products join

select /*+ leading(sales products) use_hash(products)
use_nl(customers) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

spool off
